require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_masterpass_merchant/models/checkout'
 

module MastercardMasterpassMerchant
 module Api

  class CheckoutApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMasterpassMerchant::Tracker


    # Check out service
    # This service is used to retrieve consumer&#39;s payment, shipping address, reward and 3-D Secure information from MasterPass. The checkout resource URL supplied by MasterPass must be decoded and consumed by the merchant as provided by MasterPass.
    # @param pathid Checkout identifier.
    # @param oauth_token Access token, which is used to retrieve checkout data.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [Checkout]
    def self.show(pathid, oauth_token, api_config = nil)
		 path = "/masterpass/v6/checkout/{pathid}"
	 	 service_request = ServiceRequest.new
     service_request.path_params["pathid"] = pathid
	 
	   service_request.headers["oauth_token"] = oauth_token
	 
	 
	   service_request.content_type = "application/xml"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "GET",Checkout)
    end

  end

 end
end



