require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_masterpass_merchant/models/merchant_initialization_response'
require_relative '../../mastercard_masterpass_merchant/models/merchant_initialization_request'
 

module MastercardMasterpassMerchant
 module Api

  class MerchantInitializationApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMasterpassMerchant::Tracker


    # Merchant Initialization Service
    # This service is used to secure Lightbox connections between merchant and MasterPass. This service requires a request token (OAuthToken). This service call should be used when shopping cart service is not called, for example, pairing outside of checkout flow.
    # @param merchant_initialization_request Merchant Initialization request.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [MerchantInitializationResponse]
    def self.create(merchant_initialization_request, api_config = nil)
		 path = "/masterpass/v6/merchant-initialization"
	 	 service_request = ServiceRequest.new
     
	   
	   service_request.body = merchant_initialization_request
	 
	   service_request.content_type = "application/xml"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "POST",MerchantInitializationResponse)
    end

  end

 end
end



