require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_masterpass_merchant/models/precheckout_data_response'
require_relative '../../mastercard_masterpass_merchant/models/precheckout_data_request'
 

module MastercardMasterpassMerchant
 module Api

  class PrecheckoutDataApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMasterpassMerchant::Tracker


    # Precheckout Data Service.
    # MasterPass provides merchants with the ability to request a paired consumer&#39;s data (card alias, shipping addresses, reward program, and profile information) prior to the actual MasterPass checkout. This gives the merchant the ability to provide the consumer the opportunity to preselect their checkout options before completing the checkout. If, for any reason, the precheckout call gets rejected at MasterPass (merchant requests data that the consumer did not originally consent to, if the pairing has been deleted by the user, if the Long Access token has expired, and so on), the merchant has to request pairing again.
    # @param oauth_token Long access token, which is used to retrieve precheckout data.
    # @param precheckout_data_request PreCheckout Data Request.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [PrecheckoutDataResponse]
    def self.create(oauth_token, precheckout_data_request, api_config = nil)
		 path = "/masterpass/v6/precheckout"
	 	 service_request = ServiceRequest.new
     
	   service_request.headers["oauth_token"] = oauth_token
	 
	   service_request.body = precheckout_data_request
	 
	   service_request.content_type = "application/xml"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "POST",PrecheckoutDataResponse)
    end

  end

 end
end



