require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/authentication_options'
require_relative '../../mastercard_masterpass_merchant/models/card'
require_relative '../../mastercard_masterpass_merchant/models/checkout_extension'
require_relative '../../mastercard_masterpass_merchant/models/contact'
require_relative '../../mastercard_masterpass_merchant/models/reward_program'
require_relative '../../mastercard_masterpass_merchant/models/shipping_address'
 

module MastercardMasterpassMerchant
  # This class contains various methods for to set the different values like card, contact, shipping address, authentication options, reward program details.
  class Checkout
    include ROXML
    
    xml_name "Checkout"
    
    # @!attribute card
    # @return [Card] the card details.
    xml_accessor :card, :from =>"Card",:as => Card
   
    # @!attribute transaction_id
    # @return [String] the transaction id.
    xml_accessor :transaction_id, :from =>"TransactionId"
   
    # @!attribute contact
    # @return [Contact] the contact details.
    xml_accessor :contact, :from =>"Contact",:as => Contact
   
    # @!attribute shipping_address
    # @return [ShippingAddress] the shipping address details.
    xml_accessor :shipping_address, :from =>"ShippingAddress",:as => ShippingAddress
   
    # @!attribute authentication_options
    # @return [AuthenticationOptions] the authentication options.
    xml_accessor :authentication_options, :from =>"AuthenticationOptions",:as => AuthenticationOptions
   
    # @!attribute reward_program
    # @return [RewardProgram] the reward program details.
    xml_accessor :reward_program, :from =>"RewardProgram",:as => RewardProgram
   
    # @!attribute wallet_id
    # @return [String] the value which helps to identify origin wallet.
    xml_accessor :wallet_id, :from =>"WalletID"
   
    # @!attribute pre_checkout_transaction_id
    # @return [String] the ID associated with the PreCheckout Transaction.
    xml_accessor :pre_checkout_transaction_id, :from =>"PreCheckoutTransactionId"
   
    # @!attribute extension_point
    # @return [CheckoutExtension] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => CheckoutExtension
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :card => :Card ,
        :transaction_id => :TransactionId ,
        :contact => :Contact ,
        :shipping_address => :ShippingAddress ,
        :authentication_options => :AuthenticationOptions ,
        :reward_program => :RewardProgram ,
        :wallet_id => :WalletID ,
        :pre_checkout_transaction_id => :PreCheckoutTransactionId ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:Card) ||  attributes.has_key?(:card)
        self.card = attributes[:Card] || attributes[:card]
      end
      
      if attributes.has_key?(:TransactionId) ||  attributes.has_key?(:transaction_id)
        self.transaction_id = attributes[:TransactionId] || attributes[:transaction_id]
      end
      
      if attributes.has_key?(:Contact) ||  attributes.has_key?(:contact)
        self.contact = attributes[:Contact] || attributes[:contact]
      end
      
      if attributes.has_key?(:ShippingAddress) ||  attributes.has_key?(:shipping_address)
        self.shipping_address = attributes[:ShippingAddress] || attributes[:shipping_address]
      end
      
      if attributes.has_key?(:AuthenticationOptions) ||  attributes.has_key?(:authentication_options)
        self.authentication_options = attributes[:AuthenticationOptions] || attributes[:authentication_options]
      end
      
      if attributes.has_key?(:RewardProgram) ||  attributes.has_key?(:reward_program)
        self.reward_program = attributes[:RewardProgram] || attributes[:reward_program]
      end
      
      if attributes.has_key?(:WalletID) ||  attributes.has_key?(:wallet_id)
        self.wallet_id = attributes[:WalletID] || attributes[:wallet_id]
      end
      
      if attributes.has_key?(:PreCheckoutTransactionId) ||  attributes.has_key?(:pre_checkout_transaction_id)
        self.pre_checkout_transaction_id = attributes[:PreCheckoutTransactionId] || attributes[:pre_checkout_transaction_id]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          card == o.card &&
          transaction_id == o.transaction_id &&
          contact == o.contact &&
          shipping_address == o.shipping_address &&
          authentication_options == o.authentication_options &&
          reward_program == o.reward_program &&
          wallet_id == o.wallet_id &&
          pre_checkout_transaction_id == o.pre_checkout_transaction_id &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [card, transaction_id, contact, shipping_address, authentication_options, reward_program, wallet_id, pre_checkout_transaction_id, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :card => 'Card',
        :transaction_id => 'String',
        :contact => 'Contact',
        :shipping_address => 'ShippingAddress',
        :authentication_options => 'AuthenticationOptions',
        :reward_program => 'RewardProgram',
        :wallet_id => 'String',
        :pre_checkout_transaction_id => 'String',
        :extension_point => 'CheckoutExtension'
        
      }
    end
  end


end
