require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/checkout_dsrp'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
require_relative '../../mastercard_masterpass_merchant/models/installment_option'
require_relative '../../mastercard_masterpass_merchant/models/private_datas'
require_relative '../../mastercard_masterpass_merchant/models/risk_decisioning'
require_relative '../../mastercard_masterpass_merchant/models/tokenization'
 

module MastercardMasterpassMerchant
  # This class contains methods require to set checkout details during DSRP.
  class CheckoutExtension
    include ROXML
    
    xml_name "CheckoutExtension"
    
    # @!attribute card_verification_status
    # @return [String] the card verification status.
    xml_accessor :card_verification_status, :from =>"CardVerificationStatus"
   
    # @!attribute login_authentication_method
    # @return [String] the login authentication method.
    xml_accessor :login_authentication_method, :from =>"LoginAuthenticationMethod"
   
    # @!attribute risk_decisioning
    # @return [RiskDecisioning] the risk decisioning data.
    xml_accessor :risk_decisioning, :from =>"RiskDecisioning",:as => RiskDecisioning
   
    # @!attribute dsrp
    # @return [CheckoutDSRP] the DSRP data.
    xml_accessor :dsrp, :from =>"DSRP",:as => CheckoutDSRP
   
    # @!attribute private_datas
    # @return [PrivateDatas] the private data details.
    xml_accessor :private_datas, :from =>"PrivateDatas",:as => PrivateDatas
   
    # @!attribute installment_option
    # @return [InstallmentOption] the installment options details.
    xml_accessor :installment_option, :from =>"InstallmentOption",:as => InstallmentOption
   
    # @!attribute tokenization
    # @return [Tokenization] the token details.
    xml_accessor :tokenization, :from =>"Tokenization",:as => Tokenization
   
    # @!attribute payment_account_reference
    # @return [String] the payment account reference.
    xml_accessor :payment_account_reference, :from =>"PaymentAccountReference"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :card_verification_status => :CardVerificationStatus ,
        :login_authentication_method => :LoginAuthenticationMethod ,
        :risk_decisioning => :RiskDecisioning ,
        :dsrp => :DSRP ,
        :private_datas => :PrivateDatas ,
        :installment_option => :InstallmentOption ,
        :tokenization => :Tokenization ,
        :payment_account_reference => :PaymentAccountReference ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:CardVerificationStatus) ||  attributes.has_key?(:card_verification_status)
        self.card_verification_status = attributes[:CardVerificationStatus] || attributes[:card_verification_status]
      end
      
      if attributes.has_key?(:LoginAuthenticationMethod) ||  attributes.has_key?(:login_authentication_method)
        self.login_authentication_method = attributes[:LoginAuthenticationMethod] || attributes[:login_authentication_method]
      end
      
      if attributes.has_key?(:RiskDecisioning) ||  attributes.has_key?(:risk_decisioning)
        self.risk_decisioning = attributes[:RiskDecisioning] || attributes[:risk_decisioning]
      end
      
      if attributes.has_key?(:DSRP) ||  attributes.has_key?(:dsrp)
        self.dsrp = attributes[:DSRP] || attributes[:dsrp]
      end
      
      if attributes.has_key?(:PrivateDatas) ||  attributes.has_key?(:private_datas)
        self.private_datas = attributes[:PrivateDatas] || attributes[:private_datas]
      end
      
      if attributes.has_key?(:InstallmentOption) ||  attributes.has_key?(:installment_option)
        self.installment_option = attributes[:InstallmentOption] || attributes[:installment_option]
      end
      
      if attributes.has_key?(:Tokenization) ||  attributes.has_key?(:tokenization)
        self.tokenization = attributes[:Tokenization] || attributes[:tokenization]
      end
      
      if attributes.has_key?(:PaymentAccountReference) ||  attributes.has_key?(:payment_account_reference)
        self.payment_account_reference = attributes[:PaymentAccountReference] || attributes[:payment_account_reference]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          card_verification_status == o.card_verification_status &&
          login_authentication_method == o.login_authentication_method &&
          risk_decisioning == o.risk_decisioning &&
          dsrp == o.dsrp &&
          private_datas == o.private_datas &&
          installment_option == o.installment_option &&
          tokenization == o.tokenization &&
          payment_account_reference == o.payment_account_reference &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [card_verification_status, login_authentication_method, risk_decisioning, dsrp, private_datas, installment_option, tokenization, payment_account_reference, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :card_verification_status => 'String',
        :login_authentication_method => 'String',
        :risk_decisioning => 'RiskDecisioning',
        :dsrp => 'CheckoutDSRP',
        :private_datas => 'PrivateDatas',
        :installment_option => 'InstallmentOption',
        :tokenization => 'Tokenization',
        :payment_account_reference => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
