require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/date_of_birth'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
 

module MastercardMasterpassMerchant
  # This class contains methods to get consumer contact information.
  class Contact
    include ROXML
    
    xml_name "Contact"
    
    # @!attribute first_name
    # @return [String] the contact first name.
    xml_accessor :first_name, :from =>"FirstName"
   
    # @!attribute middle_name
    # @return [String] the contact middle name or initial.
    xml_accessor :middle_name, :from =>"MiddleName"
   
    # @!attribute last_name
    # @return [String] the contact surname.
    xml_accessor :last_name, :from =>"LastName"
   
    # @!attribute gender
    # @return [String] the contact Gender (M or F). NOTE: This field may only be requested from a MasterPass wallet if it is required by law in a region. Merchants and service providers seeking to use this field must work with the local MasterPass representative to get the necessary clearances before requesting these data elements.
    xml_accessor :gender, :from =>"Gender"
   
    # @!attribute date_of_birth
    # @return [DateOfBirth] the contact DOB - YYYY/MM/DD. NOTE: This field may only be requested from a MasterPass wallet if it is required by law in a region. Merchants and service providers seeking to use this field must work with the local MasterPass representative to get the necessary clearances before requesting these data elements.
    xml_accessor :date_of_birth, :from =>"DateOfBirth",:as => DateOfBirth
   
    # @!attribute national_id
    # @return [String] the contact National Identification. NOTE: This field may only be requested from a MasterPass wallet if it is required by law in a region. Merchants and service providers seeking to use this field must work with the local MasterPass representative to get the necessary clearances before requesting these data elements.
    xml_accessor :national_id, :from =>"NationalID"
   
    # @!attribute country
    # @return [String] the contact country of residence.
    xml_accessor :country, :from =>"Country"
   
    # @!attribute email_address
    # @return [String] the contact email aAddress.
    xml_accessor :email_address, :from =>"EmailAddress"
   
    # @!attribute phone_number
    # @return [String] the contact phone.
    xml_accessor :phone_number, :from =>"PhoneNumber"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :first_name => :FirstName ,
        :middle_name => :MiddleName ,
        :last_name => :LastName ,
        :gender => :Gender ,
        :date_of_birth => :DateOfBirth ,
        :national_id => :NationalID ,
        :country => :Country ,
        :email_address => :EmailAddress ,
        :phone_number => :PhoneNumber ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:FirstName) ||  attributes.has_key?(:first_name)
        self.first_name = attributes[:FirstName] || attributes[:first_name]
      end
      
      if attributes.has_key?(:MiddleName) ||  attributes.has_key?(:middle_name)
        self.middle_name = attributes[:MiddleName] || attributes[:middle_name]
      end
      
      if attributes.has_key?(:LastName) ||  attributes.has_key?(:last_name)
        self.last_name = attributes[:LastName] || attributes[:last_name]
      end
      
      if attributes.has_key?(:Gender) ||  attributes.has_key?(:gender)
        self.gender = attributes[:Gender] || attributes[:gender]
      end
      
      if attributes.has_key?(:DateOfBirth) ||  attributes.has_key?(:date_of_birth)
        self.date_of_birth = attributes[:DateOfBirth] || attributes[:date_of_birth]
      end
      
      if attributes.has_key?(:NationalID) ||  attributes.has_key?(:national_id)
        self.national_id = attributes[:NationalID] || attributes[:national_id]
      end
      
      if attributes.has_key?(:Country) ||  attributes.has_key?(:country)
        self.country = attributes[:Country] || attributes[:country]
      end
      
      if attributes.has_key?(:EmailAddress) ||  attributes.has_key?(:email_address)
        self.email_address = attributes[:EmailAddress] || attributes[:email_address]
      end
      
      if attributes.has_key?(:PhoneNumber) ||  attributes.has_key?(:phone_number)
        self.phone_number = attributes[:PhoneNumber] || attributes[:phone_number]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Custom attribute writer method checking allowed values (enum).
    def gender=(gender)
      allowed_values = ["M", "F"]
      if gender && !allowed_values.include?(gender)
        fail "invalid value for 'gender', must be one of #{allowed_values}"
      end
      @gender = gender
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          first_name == o.first_name &&
          middle_name == o.middle_name &&
          last_name == o.last_name &&
          gender == o.gender &&
          date_of_birth == o.date_of_birth &&
          national_id == o.national_id &&
          country == o.country &&
          email_address == o.email_address &&
          phone_number == o.phone_number &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [first_name, middle_name, last_name, gender, date_of_birth, national_id, country, email_address, phone_number, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :first_name => 'String',
        :middle_name => 'String',
        :last_name => 'String',
        :gender => 'String',
        :date_of_birth => 'DateOfBirth',
        :national_id => 'String',
        :country => 'String',
        :email_address => 'String',
        :phone_number => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
