require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
 

module MastercardMasterpassMerchant
  # This class contains methods require to set DSRP details during DSRP checkout.
  class DSRP
    include ROXML
    
    xml_name "DSRP"
    
    # @!attribute dsrp_data
    # @return [String] the DSRP cryptogram generated by the consumers MasterPass wallet.
    xml_accessor :dsrp_data, :from =>"DSRPData"
   
    # @!attribute dsrp_data_type
    # @return [String] the type of cryptogram generated by the consumers MasterPass wallet. MasterPass passes the most secure selection (ICC) if the merchant or service provider has indicated they can accept both types (UCAF, ICC).
    xml_accessor :dsrp_data_type, :from =>"DSRPDataType"
   
    # @!attribute unpredictable_number
    # @return [String] the encoded EMV-quality random number generated by either the merchant or MasterPass.
    xml_accessor :unpredictable_number, :from =>"UnpredictableNumber"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :dsrp_data => :DSRPData ,
        :dsrp_data_type => :DSRPDataType ,
        :unpredictable_number => :UnpredictableNumber ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:DSRPData) ||  attributes.has_key?(:dsrp_data)
        self.dsrp_data = attributes[:DSRPData] || attributes[:dsrp_data]
      end
      
      if attributes.has_key?(:DSRPDataType) ||  attributes.has_key?(:dsrp_data_type)
        self.dsrp_data_type = attributes[:DSRPDataType] || attributes[:dsrp_data_type]
      end
      
      if attributes.has_key?(:UnpredictableNumber) ||  attributes.has_key?(:unpredictable_number)
        self.unpredictable_number = attributes[:UnpredictableNumber] || attributes[:unpredictable_number]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          dsrp_data == o.dsrp_data &&
          dsrp_data_type == o.dsrp_data_type &&
          unpredictable_number == o.unpredictable_number &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [dsrp_data, dsrp_data_type, unpredictable_number, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :dsrp_data => 'String',
        :dsrp_data_type => 'String',
        :unpredictable_number => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
