require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
 

module MastercardMasterpassMerchant
  # This class contains methods require for express checkout request.
  class ExpressCheckoutRequest
    include ROXML
    
    xml_name "ExpressCheckoutRequest"
    
    # @!attribute merchant_checkout_id
    # @return [String] the merchant checkout id.
    xml_accessor :merchant_checkout_id, :from =>"MerchantCheckoutId"
   
    # @!attribute precheckout_transaction_id
    # @return [String] the unique id of the transaction.
    xml_accessor :precheckout_transaction_id, :from =>"PrecheckoutTransactionId"
   
    # @!attribute currency_code
    # @return [String] the currency code as ISO 4217 standard (for example, USD).
    xml_accessor :currency_code, :from =>"CurrencyCode"
   
    # @!attribute order_amount
    # @return [Integer] the order amount of the transaction.
    xml_accessor :order_amount, :from =>"OrderAmount"
   
    # @!attribute card_id
    # @return [String] the id assigned to the card.
    xml_accessor :card_id, :from =>"CardId"
   
    # @!attribute digital_goods
    # @return [BOOLEAN] the digital goods flag. Set to true if it's a digital good and no shipping is required (for example, digital music, online games, and so on); optional.
    xml_accessor :digital_goods, :from =>"DigitalGoods"
   
    # @!attribute shipping_address_id
    # @return [String] the id assigned to the shipping address.
    xml_accessor :shipping_address_id, :from =>"ShippingAddressId"
   
    # @!attribute reward_program_id
    # @return [String] the id assigned to the reward program.
    xml_accessor :reward_program_id, :from =>"RewardProgramId"
   
    # @!attribute advanced_checkout_override
    # @return [BOOLEAN] the advanced checkout override flag. Set to true to override and disable 3-D Secure for current transaction.
    xml_accessor :advanced_checkout_override, :from =>"AdvancedCheckoutOverride"
   
    # @!attribute origin_url
    # @return [String] the URL that the merchant is using to initialize the Lightbox.
    xml_accessor :origin_url, :from =>"OriginUrl"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :merchant_checkout_id => :MerchantCheckoutId ,
        :precheckout_transaction_id => :PrecheckoutTransactionId ,
        :currency_code => :CurrencyCode ,
        :order_amount => :OrderAmount ,
        :card_id => :CardId ,
        :digital_goods => :DigitalGoods ,
        :shipping_address_id => :ShippingAddressId ,
        :reward_program_id => :RewardProgramId ,
        :advanced_checkout_override => :AdvancedCheckoutOverride ,
        :origin_url => :OriginUrl ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:MerchantCheckoutId) ||  attributes.has_key?(:merchant_checkout_id)
        self.merchant_checkout_id = attributes[:MerchantCheckoutId] || attributes[:merchant_checkout_id]
      end
      
      if attributes.has_key?(:PrecheckoutTransactionId) ||  attributes.has_key?(:precheckout_transaction_id)
        self.precheckout_transaction_id = attributes[:PrecheckoutTransactionId] || attributes[:precheckout_transaction_id]
      end
      
      if attributes.has_key?(:CurrencyCode) ||  attributes.has_key?(:currency_code)
        self.currency_code = attributes[:CurrencyCode] || attributes[:currency_code]
      end
      
      if attributes.has_key?(:OrderAmount) ||  attributes.has_key?(:order_amount)
        self.order_amount = attributes[:OrderAmount] || attributes[:order_amount]
      end
      
      if attributes.has_key?(:CardId) ||  attributes.has_key?(:card_id)
        self.card_id = attributes[:CardId] || attributes[:card_id]
      end
      
      if attributes.has_key?(:DigitalGoods) ||  attributes.has_key?(:digital_goods)
        self.digital_goods = attributes[:DigitalGoods] || attributes[:digital_goods]
      end
      
      if attributes.has_key?(:ShippingAddressId) ||  attributes.has_key?(:shipping_address_id)
        self.shipping_address_id = attributes[:ShippingAddressId] || attributes[:shipping_address_id]
      end
      
      if attributes.has_key?(:RewardProgramId) ||  attributes.has_key?(:reward_program_id)
        self.reward_program_id = attributes[:RewardProgramId] || attributes[:reward_program_id]
      end
      
      if attributes.has_key?(:AdvancedCheckoutOverride) ||  attributes.has_key?(:advanced_checkout_override)
        self.advanced_checkout_override = attributes[:AdvancedCheckoutOverride] || attributes[:advanced_checkout_override]
      end
      
      if attributes.has_key?(:OriginUrl) ||  attributes.has_key?(:origin_url)
        self.origin_url = attributes[:OriginUrl] || attributes[:origin_url]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          merchant_checkout_id == o.merchant_checkout_id &&
          precheckout_transaction_id == o.precheckout_transaction_id &&
          currency_code == o.currency_code &&
          order_amount == o.order_amount &&
          card_id == o.card_id &&
          digital_goods == o.digital_goods &&
          shipping_address_id == o.shipping_address_id &&
          reward_program_id == o.reward_program_id &&
          advanced_checkout_override == o.advanced_checkout_override &&
          origin_url == o.origin_url &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [merchant_checkout_id, precheckout_transaction_id, currency_code, order_amount, card_id, digital_goods, shipping_address_id, reward_program_id, advanced_checkout_override, origin_url, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :merchant_checkout_id => 'String',
        :precheckout_transaction_id => 'String',
        :currency_code => 'String',
        :order_amount => 'Integer',
        :card_id => 'String',
        :digital_goods => 'BOOLEAN',
        :shipping_address_id => 'String',
        :reward_program_id => 'String',
        :advanced_checkout_override => 'BOOLEAN',
        :origin_url => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
