require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/brands'
require_relative '../../mastercard_masterpass_merchant/models/installment_options'
require_relative '../../mastercard_masterpass_merchant/models/issuers'
 

module MastercardMasterpassMerchant
  # This class contains methods regarding installment.
  class Installment
    include ROXML
    
    xml_name "Installment"
    
    # @!attribute brands
    # @return [Brands] the brand details.
    xml_accessor :brands, :from =>"Brands",:as => Brands
   
    # @!attribute issuers
    # @return [Issuers] the issuers details.
    xml_accessor :issuers, :from =>"Issuers",:as => Issuers
   
    # @!attribute currency_code
    # @return [String] the currency code.
    xml_accessor :currency_code, :from =>"CurrencyCode"
   
    # @!attribute shipping_calculation
    # @return [String] the shipping calculation types.
    xml_accessor :shipping_calculation, :from =>"ShippingCalculation"
   
    # @!attribute installment_options
    # @return [InstallmentOptions] the installment options.
    xml_accessor :installment_options, :from =>"InstallmentOptions",:as => InstallmentOptions
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :brands => :Brands ,
        :issuers => :Issuers ,
        :currency_code => :CurrencyCode ,
        :shipping_calculation => :ShippingCalculation ,
        :installment_options => :InstallmentOptions 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:Brands) ||  attributes.has_key?(:brands)
        self.brands = attributes[:Brands] || attributes[:brands]
      end
      
      if attributes.has_key?(:Issuers) ||  attributes.has_key?(:issuers)
        self.issuers = attributes[:Issuers] || attributes[:issuers]
      end
      
      if attributes.has_key?(:CurrencyCode) ||  attributes.has_key?(:currency_code)
        self.currency_code = attributes[:CurrencyCode] || attributes[:currency_code]
      end
      
      if attributes.has_key?(:ShippingCalculation) ||  attributes.has_key?(:shipping_calculation)
        self.shipping_calculation = attributes[:ShippingCalculation] || attributes[:shipping_calculation]
      end
      
      if attributes.has_key?(:InstallmentOptions) ||  attributes.has_key?(:installment_options)
        self.installment_options = attributes[:InstallmentOptions] || attributes[:installment_options]
      end
      
    end
    
 
	

    # Custom attribute writer method checking allowed values (enum).
    def shipping_calculation=(shipping_calculation)
      allowed_values = ["FREE_SHIPPING", "SHIPPING_INCLUDED", "SHIPPING_NOT_INCLUDED"]
      if shipping_calculation && !allowed_values.include?(shipping_calculation)
        fail "invalid value for 'shipping_calculation', must be one of #{allowed_values}"
      end
      @shipping_calculation = shipping_calculation
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          brands == o.brands &&
          issuers == o.issuers &&
          currency_code == o.currency_code &&
          shipping_calculation == o.shipping_calculation &&
          installment_options == o.installment_options
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [brands, issuers, currency_code, shipping_calculation, installment_options].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :brands => 'Brands',
        :issuers => 'Issuers',
        :currency_code => 'String',
        :shipping_calculation => 'String',
        :installment_options => 'InstallmentOptions'
        
      }
    end
  end


end
