require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/merchant_initialization_extension'
 

module MastercardMasterpassMerchant
  # This class contains various methods for to set different merchant initialization request parameters required for Merchant Initialization Service.
  class MerchantInitializationRequest
    include ROXML
    
    xml_name "MerchantInitializationRequest"
    
    # @!attribute o_auth_token
    # @return [String] the Request Token (oauth_token) returned by call to the request_token API.
    xml_accessor :o_auth_token, :from =>"OAuthToken"
   
    # @!attribute pre_checkout_transaction_id
    # @return [String] the precheckout transaction identifier. Returned from get precheckout data call.
    xml_accessor :pre_checkout_transaction_id, :from =>"PreCheckoutTransactionId"
   
    # @!attribute origin_url
    # @return [String] the URL of the page that will initialize the Lightbox.
    xml_accessor :origin_url, :from =>"OriginUrl"
   
    # @!attribute extension_point
    # @return [MerchantInitializationExtension] the MerchantInitializationExtension for DSRP/future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => MerchantInitializationExtension
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :o_auth_token => :OAuthToken ,
        :pre_checkout_transaction_id => :PreCheckoutTransactionId ,
        :origin_url => :OriginUrl ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:OAuthToken) ||  attributes.has_key?(:o_auth_token)
        self.o_auth_token = attributes[:OAuthToken] || attributes[:o_auth_token]
      end
      
      if attributes.has_key?(:PreCheckoutTransactionId) ||  attributes.has_key?(:pre_checkout_transaction_id)
        self.pre_checkout_transaction_id = attributes[:PreCheckoutTransactionId] || attributes[:pre_checkout_transaction_id]
      end
      
      if attributes.has_key?(:OriginUrl) ||  attributes.has_key?(:origin_url)
        self.origin_url = attributes[:OriginUrl] || attributes[:origin_url]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          o_auth_token == o.o_auth_token &&
          pre_checkout_transaction_id == o.pre_checkout_transaction_id &&
          origin_url == o.origin_url &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [o_auth_token, pre_checkout_transaction_id, origin_url, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :o_auth_token => 'String',
        :pre_checkout_transaction_id => 'String',
        :origin_url => 'String',
        :extension_point => 'MerchantInitializationExtension'
        
      }
    end
  end


end
