require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
 

module MastercardMasterpassMerchant
  # This class contains different methods for to send and retrieve transaction details.
  class MerchantTransaction
    include ROXML
    
    xml_name "MerchantTransaction"
    
    # @!attribute transaction_id
    # @return [String] the transaction Id.
    xml_accessor :transaction_id, :from =>"TransactionId"
   
    # @!attribute consumer_key
    # @return [String] the consumer key from checkout project.
    xml_accessor :consumer_key, :from =>"ConsumerKey"
   
    # @!attribute currency
    # @return [String] the currency for the transaction, for example, USD.
    xml_accessor :currency, :from =>"Currency"
   
    # @!attribute order_amount
    # @return [Integer] the transaction order amount, for example, 1500 (for USD 15 transaction amount)
    xml_accessor :order_amount, :from =>"OrderAmount"
   
    # @!attribute purchase_date
    # @return [String] the date of purchase.
    xml_accessor :purchase_date, :from =>"PurchaseDate"
   
    # @!attribute transaction_status
    # @return [String] the status of transaction. Valid values are, SUCCESS: For approved transaction and FAILURE: For declined transaction.
    xml_accessor :transaction_status, :from =>"TransactionStatus"
   
    # @!attribute approval_code
    # @return [String] the six-digit approval code returned by payment API.
    xml_accessor :approval_code, :from =>"ApprovalCode"
   
    # @!attribute pre_checkout_transaction_id
    # @return [String] the precheckout transaction id element of the PrecheckoutData.(This is not required for Standard Checkout)
    xml_accessor :pre_checkout_transaction_id, :from =>"PreCheckoutTransactionId"
   
    # @!attribute express_checkout_indicator
    # @return [BOOLEAN] the express checkout indicator. Set to true for express checkout
    xml_accessor :express_checkout_indicator, :from =>"ExpressCheckoutIndicator"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :transaction_id => :TransactionId ,
        :consumer_key => :ConsumerKey ,
        :currency => :Currency ,
        :order_amount => :OrderAmount ,
        :purchase_date => :PurchaseDate ,
        :transaction_status => :TransactionStatus ,
        :approval_code => :ApprovalCode ,
        :pre_checkout_transaction_id => :PreCheckoutTransactionId ,
        :express_checkout_indicator => :ExpressCheckoutIndicator ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:TransactionId) ||  attributes.has_key?(:transaction_id)
        self.transaction_id = attributes[:TransactionId] || attributes[:transaction_id]
      end
      
      if attributes.has_key?(:ConsumerKey) ||  attributes.has_key?(:consumer_key)
        self.consumer_key = attributes[:ConsumerKey] || attributes[:consumer_key]
      end
      
      if attributes.has_key?(:Currency) ||  attributes.has_key?(:currency)
        self.currency = attributes[:Currency] || attributes[:currency]
      end
      
      if attributes.has_key?(:OrderAmount) ||  attributes.has_key?(:order_amount)
        self.order_amount = attributes[:OrderAmount] || attributes[:order_amount]
      end
      
      if attributes.has_key?(:PurchaseDate) ||  attributes.has_key?(:purchase_date)
        self.purchase_date = attributes[:PurchaseDate] || attributes[:purchase_date]
      end
      
      if attributes.has_key?(:TransactionStatus) ||  attributes.has_key?(:transaction_status)
        self.transaction_status = attributes[:TransactionStatus] || attributes[:transaction_status]
      end
      
      if attributes.has_key?(:ApprovalCode) ||  attributes.has_key?(:approval_code)
        self.approval_code = attributes[:ApprovalCode] || attributes[:approval_code]
      end
      
      if attributes.has_key?(:PreCheckoutTransactionId) ||  attributes.has_key?(:pre_checkout_transaction_id)
        self.pre_checkout_transaction_id = attributes[:PreCheckoutTransactionId] || attributes[:pre_checkout_transaction_id]
      end
      
      if attributes.has_key?(:ExpressCheckoutIndicator) ||  attributes.has_key?(:express_checkout_indicator)
        self.express_checkout_indicator = attributes[:ExpressCheckoutIndicator] || attributes[:express_checkout_indicator]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Custom attribute writer method checking allowed values (enum).
    def transaction_status=(transaction_status)
      allowed_values = ["Success", "Failure"]
      if transaction_status && !allowed_values.include?(transaction_status)
        fail "invalid value for 'transaction_status', must be one of #{allowed_values}"
      end
      @transaction_status = transaction_status
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          transaction_id == o.transaction_id &&
          consumer_key == o.consumer_key &&
          currency == o.currency &&
          order_amount == o.order_amount &&
          purchase_date == o.purchase_date &&
          transaction_status == o.transaction_status &&
          approval_code == o.approval_code &&
          pre_checkout_transaction_id == o.pre_checkout_transaction_id &&
          express_checkout_indicator == o.express_checkout_indicator &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [transaction_id, consumer_key, currency, order_amount, purchase_date, transaction_status, approval_code, pre_checkout_transaction_id, express_checkout_indicator, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :transaction_id => 'String',
        :consumer_key => 'String',
        :currency => 'String',
        :order_amount => 'Integer',
        :purchase_date => 'String',
        :transaction_status => 'String',
        :approval_code => 'String',
        :pre_checkout_transaction_id => 'String',
        :express_checkout_indicator => 'BOOLEAN',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
