require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/address'
require_relative '../../mastercard_masterpass_merchant/models/base_card'
require_relative '../../mastercard_masterpass_merchant/models/card_extension'
 

module MastercardMasterpassMerchant
  class PrecheckoutCard
    include ROXML
    
    xml_name "PrecheckoutCard"
    
    # @!attribute last_four
    # @return [String] the last four digits of card.
    xml_accessor :last_four, :from =>"LastFour"
   
    # @!attribute card_id
    # @return [String] the card id.
    xml_accessor :card_id, :from =>"CardId"
   
    # @!attribute brand_id
    # @return [String] the card brand id, for example, master for MasterCard.
    xml_accessor :brand_id, :from =>"BrandId"
   
    # @!attribute card_alias
    # @return [String] the card alias.
    xml_accessor :card_alias, :from =>"CardAlias"
   
    # @!attribute expiry_month
    # @return [Integer] the expiration month displayed on the payment card.
    xml_accessor :expiry_month, :from =>"ExpiryMonth"
   
    # @!attribute selected_as_default
    # @return [BOOLEAN] the default selection.
    xml_accessor :selected_as_default, :from =>"SelectedAsDefault"
   
    # @!attribute bnb_unverified
    # @return [BOOLEAN] the flag for BNB unverified.
    xml_accessor :bnb_unverified, :from =>"BNBUnverified"
   
    # @!attribute card_holder_name
    # @return [String] the cardholder name.
    xml_accessor :card_holder_name, :from =>"CardHolderName"
   
    # @!attribute extension_point
    # @return [CardExtension] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => CardExtension
   
    # @!attribute billing_address
    # @return [Address] the billing address for the card holder.
    xml_accessor :billing_address, :from =>"BillingAddress",:as => Address
   
    # @!attribute brand_name
    # @return [String] the card brand name, for example, MasterCard.
    xml_accessor :brand_name, :from =>"BrandName"
   
    # @!attribute expiry_year
    # @return [Integer] the expiration year displayed on the payment card.
    xml_accessor :expiry_year, :from =>"ExpiryYear"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :last_four => :LastFour ,
        :card_id => :CardId ,
        :brand_id => :BrandId ,
        :card_alias => :CardAlias ,
        :expiry_month => :ExpiryMonth ,
        :selected_as_default => :SelectedAsDefault ,
        :bnb_unverified => :BNBUnverified ,
        :card_holder_name => :CardHolderName ,
        :extension_point => :ExtensionPoint ,
        :billing_address => :BillingAddress ,
        :brand_name => :BrandName ,
        :expiry_year => :ExpiryYear 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:LastFour) ||  attributes.has_key?(:last_four)
        self.last_four = attributes[:LastFour] || attributes[:last_four]
      end
      
      if attributes.has_key?(:CardId) ||  attributes.has_key?(:card_id)
        self.card_id = attributes[:CardId] || attributes[:card_id]
      end
      
      if attributes.has_key?(:BrandId) ||  attributes.has_key?(:brand_id)
        self.brand_id = attributes[:BrandId] || attributes[:brand_id]
      end
      
      if attributes.has_key?(:CardAlias) ||  attributes.has_key?(:card_alias)
        self.card_alias = attributes[:CardAlias] || attributes[:card_alias]
      end
      
      if attributes.has_key?(:ExpiryMonth) ||  attributes.has_key?(:expiry_month)
        self.expiry_month = attributes[:ExpiryMonth] || attributes[:expiry_month]
      end
      
      if attributes.has_key?(:SelectedAsDefault) ||  attributes.has_key?(:selected_as_default)
        self.selected_as_default = attributes[:SelectedAsDefault] || attributes[:selected_as_default]
      end
      
      if attributes.has_key?(:BNBUnverified) ||  attributes.has_key?(:bnb_unverified)
        self.bnb_unverified = attributes[:BNBUnverified] || attributes[:bnb_unverified]
      end
      
      if attributes.has_key?(:CardHolderName) ||  attributes.has_key?(:card_holder_name)
        self.card_holder_name = attributes[:CardHolderName] || attributes[:card_holder_name]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
      if attributes.has_key?(:BillingAddress) ||  attributes.has_key?(:billing_address)
        self.billing_address = attributes[:BillingAddress] || attributes[:billing_address]
      end
      
      if attributes.has_key?(:BrandName) ||  attributes.has_key?(:brand_name)
        self.brand_name = attributes[:BrandName] || attributes[:brand_name]
      end
      
      if attributes.has_key?(:ExpiryYear) ||  attributes.has_key?(:expiry_year)
        self.expiry_year = attributes[:ExpiryYear] || attributes[:expiry_year]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          last_four == o.last_four &&
          card_id == o.card_id &&
          brand_id == o.brand_id &&
          card_alias == o.card_alias &&
          expiry_month == o.expiry_month &&
          selected_as_default == o.selected_as_default &&
          bnb_unverified == o.bnb_unverified &&
          card_holder_name == o.card_holder_name &&
          extension_point == o.extension_point &&
          billing_address == o.billing_address &&
          brand_name == o.brand_name &&
          expiry_year == o.expiry_year
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [last_four, card_id, brand_id, card_alias, expiry_month, selected_as_default, bnb_unverified, card_holder_name, extension_point, billing_address, brand_name, expiry_year].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :last_four => 'String',
        :card_id => 'String',
        :brand_id => 'String',
        :card_alias => 'String',
        :expiry_month => 'Integer',
        :selected_as_default => 'BOOLEAN',
        :bnb_unverified => 'BOOLEAN',
        :card_holder_name => 'String',
        :extension_point => 'CardExtension',
        :billing_address => 'Address',
        :brand_name => 'String',
        :expiry_year => 'Integer'
        
      }
    end
  end


end
