require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/contact'
require_relative '../../mastercard_masterpass_merchant/models/errors'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
require_relative '../../mastercard_masterpass_merchant/models/precheckout_cards'
require_relative '../../mastercard_masterpass_merchant/models/precheckout_shipping_addresses'
require_relative '../../mastercard_masterpass_merchant/models/reward_programs'
 

module MastercardMasterpassMerchant
  # This class contains different methods for different types of precheckout data like card, contact, shipping address and reward program details.
  class PrecheckoutData
    include ROXML
    
    xml_name "PrecheckoutData"
    
    # @!attribute cards
    # @return [PrecheckoutCards] the cards.
    xml_accessor :cards, :from =>"Cards",:as => PrecheckoutCards
   
    # @!attribute contact
    # @return [Contact] the contact details.
    xml_accessor :contact, :from =>"Contact",:as => Contact
   
    # @!attribute shipping_addresses
    # @return [PrecheckoutShippingAddresses] the shipping address details.
    xml_accessor :shipping_addresses, :from =>"ShippingAddresses",:as => PrecheckoutShippingAddresses
   
    # @!attribute reward_programs
    # @return [RewardPrograms] the reward program details.
    xml_accessor :reward_programs, :from =>"RewardPrograms",:as => RewardPrograms
   
    # @!attribute wallet_name
    # @return [String] the wallet name.
    xml_accessor :wallet_name, :from =>"WalletName"
   
    # @!attribute precheckout_transaction_id
    # @return [String] the precheckout transaction id.
    xml_accessor :precheckout_transaction_id, :from =>"PrecheckoutTransactionId"
   
    # @!attribute consumer_wallet_id
    # @return [String] the consumer wallet id.
    xml_accessor :consumer_wallet_id, :from =>"ConsumerWalletId"
   
    # @!attribute errors
    # @return [Errors] the errors information.
    xml_accessor :errors, :from =>"Errors",:as => Errors
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :cards => :Cards ,
        :contact => :Contact ,
        :shipping_addresses => :ShippingAddresses ,
        :reward_programs => :RewardPrograms ,
        :wallet_name => :WalletName ,
        :precheckout_transaction_id => :PrecheckoutTransactionId ,
        :consumer_wallet_id => :ConsumerWalletId ,
        :errors => :Errors ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:Cards) ||  attributes.has_key?(:cards)
        self.cards = attributes[:Cards] || attributes[:cards]
      end
      
      if attributes.has_key?(:Contact) ||  attributes.has_key?(:contact)
        self.contact = attributes[:Contact] || attributes[:contact]
      end
      
      if attributes.has_key?(:ShippingAddresses) ||  attributes.has_key?(:shipping_addresses)
        self.shipping_addresses = attributes[:ShippingAddresses] || attributes[:shipping_addresses]
      end
      
      if attributes.has_key?(:RewardPrograms) ||  attributes.has_key?(:reward_programs)
        self.reward_programs = attributes[:RewardPrograms] || attributes[:reward_programs]
      end
      
      if attributes.has_key?(:WalletName) ||  attributes.has_key?(:wallet_name)
        self.wallet_name = attributes[:WalletName] || attributes[:wallet_name]
      end
      
      if attributes.has_key?(:PrecheckoutTransactionId) ||  attributes.has_key?(:precheckout_transaction_id)
        self.precheckout_transaction_id = attributes[:PrecheckoutTransactionId] || attributes[:precheckout_transaction_id]
      end
      
      if attributes.has_key?(:ConsumerWalletId) ||  attributes.has_key?(:consumer_wallet_id)
        self.consumer_wallet_id = attributes[:ConsumerWalletId] || attributes[:consumer_wallet_id]
      end
      
      if attributes.has_key?(:Errors) ||  attributes.has_key?(:errors)
        self.errors = attributes[:Errors] || attributes[:errors]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          cards == o.cards &&
          contact == o.contact &&
          shipping_addresses == o.shipping_addresses &&
          reward_programs == o.reward_programs &&
          wallet_name == o.wallet_name &&
          precheckout_transaction_id == o.precheckout_transaction_id &&
          consumer_wallet_id == o.consumer_wallet_id &&
          errors == o.errors &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [cards, contact, shipping_addresses, reward_programs, wallet_name, precheckout_transaction_id, consumer_wallet_id, errors, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :cards => 'PrecheckoutCards',
        :contact => 'Contact',
        :shipping_addresses => 'PrecheckoutShippingAddresses',
        :reward_programs => 'RewardPrograms',
        :wallet_name => 'String',
        :precheckout_transaction_id => 'String',
        :consumer_wallet_id => 'String',
        :errors => 'Errors',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
