require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/address'
require_relative '../../mastercard_masterpass_merchant/models/address_extension_point'
 

module MastercardMasterpassMerchant
  class ShippingAddress
    include ROXML
    
    xml_name "ShippingAddress"
    
    # @!attribute recipient_phone_number
    # @return [String] the phone number of the person set to receive the shipped order.
    xml_accessor :recipient_phone_number, :from =>"RecipientPhoneNumber"
   
    # @!attribute line3
    # @return [String] the address line 3 used to enter remaining address information if it does not fit in Line 1 and Line 2.
    xml_accessor :line3, :from =>"Line3"
   
    # @!attribute line2
    # @return [String] the address line 2 used for Apt Number, Suite Number, and so on.
    xml_accessor :line2, :from =>"Line2"
   
    # @!attribute line1
    # @return [String] the address line 1 used for Street number and Street Name.
    xml_accessor :line1, :from =>"Line1"
   
    # @!attribute postal_code
    # @return [String] the postal code or zip code appended to mailing address for the purpose of sorting mail.
    xml_accessor :postal_code, :from =>"PostalCode"
   
    # @!attribute country_subdivision
    # @return [String] the cardholder's country subdivision. Defined by ISO 3166-1 alpha-2 digit code, for example, US-VA is Virginia, US-OH is Ohio
    xml_accessor :country_subdivision, :from =>"CountrySubdivision"
   
    # @!attribute recipient_name
    # @return [String] the name of person set to receive the shipped order.
    xml_accessor :recipient_name, :from =>"RecipientName"
   
    # @!attribute extension_point
    # @return [AddressExtensionPoint] the AddressExtensionPoint for additional information.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => AddressExtensionPoint
   
    # @!attribute country
    # @return [String] the cardholder's country. Defined by ISO 3166-1 alpha-2 digit country codes, for example, US is United States, AU is Australia, CA is Canada, GB is United Kingdom, and so on.
    xml_accessor :country, :from =>"Country"
   
    # @!attribute city
    # @return [String] the cardholder's city.
    xml_accessor :city, :from =>"City"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :recipient_phone_number => :RecipientPhoneNumber ,
        :line3 => :Line3 ,
        :line2 => :Line2 ,
        :line1 => :Line1 ,
        :postal_code => :PostalCode ,
        :country_subdivision => :CountrySubdivision ,
        :recipient_name => :RecipientName ,
        :extension_point => :ExtensionPoint ,
        :country => :Country ,
        :city => :City 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:RecipientPhoneNumber) ||  attributes.has_key?(:recipient_phone_number)
        self.recipient_phone_number = attributes[:RecipientPhoneNumber] || attributes[:recipient_phone_number]
      end
      
      if attributes.has_key?(:Line3) ||  attributes.has_key?(:line3)
        self.line3 = attributes[:Line3] || attributes[:line3]
      end
      
      if attributes.has_key?(:Line2) ||  attributes.has_key?(:line2)
        self.line2 = attributes[:Line2] || attributes[:line2]
      end
      
      if attributes.has_key?(:Line1) ||  attributes.has_key?(:line1)
        self.line1 = attributes[:Line1] || attributes[:line1]
      end
      
      if attributes.has_key?(:PostalCode) ||  attributes.has_key?(:postal_code)
        self.postal_code = attributes[:PostalCode] || attributes[:postal_code]
      end
      
      if attributes.has_key?(:CountrySubdivision) ||  attributes.has_key?(:country_subdivision)
        self.country_subdivision = attributes[:CountrySubdivision] || attributes[:country_subdivision]
      end
      
      if attributes.has_key?(:RecipientName) ||  attributes.has_key?(:recipient_name)
        self.recipient_name = attributes[:RecipientName] || attributes[:recipient_name]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
      if attributes.has_key?(:Country) ||  attributes.has_key?(:country)
        self.country = attributes[:Country] || attributes[:country]
      end
      
      if attributes.has_key?(:City) ||  attributes.has_key?(:city)
        self.city = attributes[:City] || attributes[:city]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          recipient_phone_number == o.recipient_phone_number &&
          line3 == o.line3 &&
          line2 == o.line2 &&
          line1 == o.line1 &&
          postal_code == o.postal_code &&
          country_subdivision == o.country_subdivision &&
          recipient_name == o.recipient_name &&
          extension_point == o.extension_point &&
          country == o.country &&
          city == o.city
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [recipient_phone_number, line3, line2, line1, postal_code, country_subdivision, recipient_name, extension_point, country, city].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :recipient_phone_number => 'String',
        :line3 => 'String',
        :line2 => 'String',
        :line1 => 'String',
        :postal_code => 'String',
        :country_subdivision => 'String',
        :recipient_name => 'String',
        :extension_point => 'AddressExtensionPoint',
        :country => 'String',
        :city => 'String'
        
      }
    end
  end


end
