require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
require_relative '../../mastercard_masterpass_merchant/models/shopping_cart_item'
 

module MastercardMasterpassMerchant
  # This class contains various methods for to set different shopping cart parameters required for Shopping Cart Service.
  class ShoppingCart
    include ROXML
    
    xml_name "ShoppingCart"
    
    # @!attribute currency_code
    # @return [String] the currency code defined by ISO 4217. Its should be exactly three characters, such as, USD for US Dollars. All MonetaryValues will be modified by the CurrencyCode.
    xml_accessor :currency_code, :from =>"CurrencyCode"
   
    # @!attribute subtotal
    # @return [Integer] the total sum of all the items in the cart excluding shipping, handling and tax. Integer without the decimal, for example, USD 119.00 will be 11900.
    xml_accessor :subtotal, :from =>"Subtotal"
   
    # @!attribute shopping_cart_item
    # @return [Array<ShoppingCartItem>] the details of a single shopping cart item.
    xml_accessor :shopping_cart_item, :from =>"ShoppingCartItem", :as =>[ShoppingCartItem]
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :currency_code => :CurrencyCode ,
        :subtotal => :Subtotal ,
        :shopping_cart_item => :ShoppingCartItem ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:CurrencyCode) ||  attributes.has_key?(:currency_code)
        self.currency_code = attributes[:CurrencyCode] || attributes[:currency_code]
      end
      
      if attributes.has_key?(:Subtotal) ||  attributes.has_key?(:subtotal)
        self.subtotal = attributes[:Subtotal] || attributes[:subtotal]
      end
      
      if attributes.has_key?(:ShoppingCartItem) ||  attributes.has_key?(:shopping_cart_item)
        self.shopping_cart_item = attributes[:ShoppingCartItem] || attributes[:shopping_cart_item]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
    # Overriding setter method shopping_cart_item
    # @param value 
  	def shopping_cart_item=(value)
      if (value.is_a?(Array))
        @shopping_cart_item = value
      else
        @shopping_cart_item = [] if !@shopping_cart_item
        @shopping_cart_item.push value
      end
    end
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          currency_code == o.currency_code &&
          subtotal == o.subtotal &&
          shopping_cart_item == o.shopping_cart_item &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [currency_code, subtotal, shopping_cart_item, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :currency_code => 'String',
        :subtotal => 'Integer',
        :shopping_cart_item => 'Array<ShoppingCartItem>',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
