require 'date'
require 'roxml'
require_relative '../../mastercard_masterpass_merchant/models/extension_point'
 

module MastercardMasterpassMerchant
  # This class contains various methods for to set shopping cart items required for Shopping Cart Service.
  class ShoppingCartItem
    include ROXML
    
    xml_name "ShoppingCartItem"
    
    # @!attribute description
    # @return [String] the description shopping cart item.
    xml_accessor :description, :from =>"Description"
   
    # @!attribute quantity
    # @return [Integer] the number of shopping cart item.
    xml_accessor :quantity, :from =>"Quantity"
   
    # @!attribute value
    # @return [Integer] the price or monetary value of a single shopping cart item. Cost * Quantity. Integer without decimal, for example, USD 100.00 is 10000.
    xml_accessor :value, :from =>"Value"
   
    # @!attribute image_url
    # @return [String] the image URL of the shopping cart item.
    xml_accessor :image_url, :from =>"ImageURL"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the ExtensionPoint for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :description => :Description ,
        :quantity => :Quantity ,
        :value => :Value ,
        :image_url => :ImageURL ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:Description) ||  attributes.has_key?(:description)
        self.description = attributes[:Description] || attributes[:description]
      end
      
      if attributes.has_key?(:Quantity) ||  attributes.has_key?(:quantity)
        self.quantity = attributes[:Quantity] || attributes[:quantity]
      end
      
      if attributes.has_key?(:Value) ||  attributes.has_key?(:value)
        self.value = attributes[:Value] || attributes[:value]
      end
      
      if attributes.has_key?(:ImageURL) ||  attributes.has_key?(:image_url)
        self.image_url = attributes[:ImageURL] || attributes[:image_url]
      end
      
      if attributes.has_key?(:ExtensionPoint) ||  attributes.has_key?(:extension_point)
        self.extension_point = attributes[:ExtensionPoint] || attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          description == o.description &&
          quantity == o.quantity &&
          value == o.value &&
          image_url == o.image_url &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [description, quantity, value, image_url, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMasterpassMerchant.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :description => 'String',
        :quantity => 'Integer',
        :value => 'Integer',
        :image_url => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
