require 'mastercard_core_sdk'

module MastercardMasterpassMerchant
  module Tracker
	# Provides tracking api information and user-agent information.
    class SdkApiTracker
	    include MastercardCoreSdk, MastercardCoreSdk::Core, MastercardCoreSdk::Tracker::ApiTracker
	  
	    # Defines the programming language.
	    #
	    # @return [String]
	    attr_accessor :language
	    
	    # Defines the programming language version.
	    #
	    # @return [String]
	    attr_accessor :language_version
	    
	    # Defines the client sdk gem version.
	    #
	    # @return [String]
	    attr_accessor :client_sdk_version
	    
	    # Defines the plugin version if any defined in MasterCardApiConfiguration as additional_properties.
	    #
	    # @return [String]
	    attr_reader :plugin_version
	
	    def initialize
	      @language = "Ruby"
	      @language_version = RUBY_VERSION
	      @client_sdk_version = MastercardMasterpassMerchant::VERSION  
	      
	      additional_track_info = MasterCardApiConfiguration.additional_properties
	      if (additional_track_info && additional_track_info.size > 0 && additional_track_info.key?(PLUGIN_VERSION))
	          @plugin_version = additional_track_info[PLUGIN_VERSION]
	      end
	    end
	    
	    # Overriding ApiTracker#tracking_info method. 
	    #
	    # @return [String]
	    def tracking_info
	      api_tracker_string = CLIENT_SDK_VERSION + client_sdk_version + SEPERATOR + LANG_NAME + language + SEPERATOR + LANG_VERSION + language_version
	      api_tracker_string += SEPERATOR + PLUGIN_VERSION + EQUAL + plugin_version unless plugin_version.nil?
	      return api_tracker_string
	    end
	    
	    # Overriding ApiTracker#user_agent_info method. 
	    #
	    # @return [String]
	    def user_agent_info
	      return USER_AGENT 
	    end
    end
  end
end
